#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
from Buttons import *
from Dialogs import *
from qubtree import Node, NullNode
from qubtree import QTR_TYPE_FLOAT
from math import *
from itertools import *

def children(tree, name=None):
    if name == None:
        child = tree.child
    else:
        child = tree.find(name)
    while not child.isNull:
        yield child
        if name == None:
            child = child.sibling
        else:
            child = child.nextSameName()

def Stability(props):
    # data
    dataFile = QUB.DataView.currData
    data = QUB.getChosenIdealization()
    dataset = data.find('DataSet')

    overlaps = Node('List')
    overlaps.data = props['listName']
    sel = NullNode()

    winsize = props['winsize']
    overlap = props['overlap']

    for seg in children(dataset, 'Segment'):    
        count = seg['DwellCount'].data[0]
        ff, ll = seg['Firsts'].data, seg['Lasts'].data
        for i in xrange(0, count-winsize, overlap):
            sel = overlaps.insert('', sel)
            sel.data = (ff[i], ll[i+winsize-1])

    if not sel.isNull:
        dataFile.setList(str(overlaps.data), [])
        dataFile.chooseList(str(overlaps.data))
        dataFile.lists.currList.setFromTree(overlaps)
        ds = QUB.GetDataSource()
        QUB.SetDataSource('list')
        QUB.Modeling['Stat']()
        QUB.SetDataSource(ds)


AddTaskButton( 'Stability', # button text
               Stability,   # function
               'Stbl',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Stability plot for idealized single-channel data",   # tool-tip
               "Stability plot for idealized single-channel data",   # description in "Edit Buttons"
               "Stability settings:", # caption for the settings dialog
               [DialogText("winsize", 'Window size (events):',50, acceptIntGreaterThan(0)),
                DialogText("overlap", 'Overlap (events):',25, acceptIntGreaterThan(0)),
                DialogText("listName", "Selection List name:", "Stbl Overlaps", acceptString)],
               'Modeling' )


